@@ -1,4 +1,6 @@
-# Copyright (C) 2001-2007, 2009 Nominum, Inc.
+# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license
+
+# Copyright (C) 2001-2017 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -15,8 +17,8 @@
 
 """DNS rdatasets (an rdataset is a set of rdatas of a given type and class)"""
 
+import io
 import random
-import StringIO
 import struct
 
 import dns.exception
@@ -28,51 +30,42 @@ import dns.set
 # define SimpleSet here for backwards compatibility
 SimpleSet = dns.set.Set
 
+
 class DifferingCovers(dns.exception.DNSException):
-    """Raised if an attempt is made to add a SIG/RRSIG whose covered type
+    """An attempt was made to add a DNS SIG/RRSIG whose covered type
     is not the same as that of the other rdatas in the rdataset."""
-    pass
+
 
 class IncompatibleTypes(dns.exception.DNSException):
-    """Raised if an attempt is made to add rdata of an incompatible type."""
-    pass
+    """An attempt was made to add DNS RR data of an incompatible type."""
+
 
 class Rdataset(dns.set.Set):
-    """A DNS rdataset.
 
-    @ivar rdclass: The class of the rdataset
-    @type rdclass: int
-    @ivar rdtype: The type of the rdataset
-    @type rdtype: int
-    @ivar covers: The covered type.  Usually this value is
-    dns.rdatatype.NONE, but if the rdtype is dns.rdatatype.SIG or
-    dns.rdatatype.RRSIG, then the covers value will be the rdata
-    type the SIG/RRSIG covers.  The library treats the SIG and RRSIG
-    types as if they were a family of
-    types, e.g. RRSIG(A), RRSIG(NS), RRSIG(SOA).  This makes RRSIGs much
-    easier to work with than if RRSIGs covering different rdata
-    types were aggregated into a single RRSIG rdataset.
-    @type covers: int
-    @ivar ttl: The DNS TTL (Time To Live) value
-    @type ttl: int
-    """
+    """A DNS rdataset."""
 
     __slots__ = ['rdclass', 'rdtype', 'covers', 'ttl']
-    
-    def __init__(self, rdclass, rdtype, covers=dns.rdatatype.NONE):
+
+    def __init__(self, rdclass, rdtype, covers=dns.rdatatype.NONE, ttl=0):
         """Create a new rdataset of the specified class and type.
 
-        @see: the description of the class instance variables for the
-        meaning of I{rdclass} and I{rdtype}"""
-        
-        super(Rdataset, self).__init__()
+        *rdclass*, an ``int``, the rdataclass.
+
+        *rdtype*, an ``int``, the rdatatype.
+
+        *covers*, an ``int``, the covered rdatatype.
+
+        *ttl*, an ``int``, the TTL.
+        """
+
+        super().__init__()
         self.rdclass = rdclass
         self.rdtype = rdtype
         self.covers = covers
-        self.ttl = 0
+        self.ttl = ttl
 
     def _clone(self):
-        obj = super(Rdataset, self)._clone()
+        obj = super()._clone()
         obj.rdclass = self.rdclass
         obj.rdtype = self.rdtype
         obj.covers = self.covers
@@ -80,12 +73,15 @@ class Rdataset(dns.set.Set):
         return obj
 
     def update_ttl(self, ttl):
-        """Set the TTL of the rdataset to be the lesser of the set's current
+        """Perform TTL minimization.
+
+        Set the TTL of the rdataset to be the lesser of the set's current
         TTL or the specified TTL.  If the set contains no rdatas, set the TTL
         to the specified TTL.
-        @param ttl: The TTL
-        @type ttl: int"""
-        
+
+        *ttl*, an ``int``.
+        """
+
         if len(self) == 0:
             self.ttl = ttl
         elif ttl < self.ttl:
@@ -94,14 +90,20 @@ class Rdataset(dns.set.Set):
     def add(self, rd, ttl=None):
         """Add the specified rdata to the rdataset.
 
-        If the optional I{ttl} parameter is supplied, then
-        self.update_ttl(ttl) will be called prior to adding the rdata.
-        
-        @param rd: The rdata
-        @type rd: dns.rdata.Rdata object
-        @param ttl: The TTL
-        @type ttl: int"""
-        
+        If the optional *ttl* parameter is supplied, then
+        ``self.update_ttl(ttl)`` will be called prior to adding the rdata.
+
+        *rd*, a ``dns.rdata.Rdata``, the rdata
+
+        *ttl*, an ``int``, the TTL.
+
+        Raises ``dns.rdataset.IncompatibleTypes`` if the type and class
+        do not match the type and class of the rdataset.
+
+        Raises ``dns.rdataset.DifferingCovers`` if the type is a signature
+        type and the covered type does not match that of the rdataset.
+        """
+
         #
         # If we're adding a signature, do some special handling to
         # check that the signature covers the same type as the
@@ -110,7 +112,7 @@ class Rdataset(dns.set.Set):
         #
         if self.rdclass != rd.rdclass or self.rdtype != rd.rdtype:
             raise IncompatibleTypes
-        if not ttl is None:
+        if ttl is not None:
             self.update_ttl(ttl)
         if self.rdtype == dns.rdatatype.RRSIG or \
            self.rdtype == dns.rdatatype.SIG:
@@ -121,24 +123,33 @@ class Rdataset(dns.set.Set):
                 raise DifferingCovers
         if dns.rdatatype.is_singleton(rd.rdtype) and len(self) > 0:
             self.clear()
-        super(Rdataset, self).add(rd)
+        super().add(rd)
 
     def union_update(self, other):
         self.update_ttl(other.ttl)
-        super(Rdataset, self).union_update(other)
+        super().union_update(other)
 
     def intersection_update(self, other):
         self.update_ttl(other.ttl)
-        super(Rdataset, self).intersection_update(other)
+        super().intersection_update(other)
 
     def update(self, other):
         """Add all rdatas in other to self.
 
-        @param other: The rdataset from which to update
-        @type other: dns.rdataset.Rdataset object"""
-        
+        *other*, a ``dns.rdataset.Rdataset``, the rdataset from which
+        to update.
+        """
+
         self.update_ttl(other.ttl)
-        super(Rdataset, self).update(other)
+        super().update(other)
+
+    def _rdata_repr(self):
+        def maybe_truncate(s):
+            if len(s) > 100:
+                return s[:100] + '...'
+            return s
+        return '[%s]' % ', '.join('<%s>' % maybe_truncate(str(rr))
+                                  for rr in self)
 
     def __repr__(self):
         if self.covers == 0:
@@ -146,24 +157,21 @@ class Rdataset(dns.set.Set):
         else:
             ctext = '(' + dns.rdatatype.to_text(self.covers) + ')'
         return '<DNS ' + dns.rdataclass.to_text(self.rdclass) + ' ' + \
-               dns.rdatatype.to_text(self.rdtype) + ctext + ' rdataset>'
-    
+               dns.rdatatype.to_text(self.rdtype) + ctext + \
+               ' rdataset: ' + self._rdata_repr() + '>'
+
     def __str__(self):
         return self.to_text()
 
     def __eq__(self, other):
-        """Two rdatasets are equal if they have the same class, type, and
-        covers, and contain the same rdata.
-        @rtype: bool"""
-        
         if not isinstance(other, Rdataset):
             return False
         if self.rdclass != other.rdclass or \
            self.rdtype != other.rdtype or \
            self.covers != other.covers:
             return False
-        return super(Rdataset, self).__eq__(other)
-    
+        return super().__eq__(other)
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
@@ -171,29 +179,32 @@ class Rdataset(dns.set.Set):
                 override_rdclass=None, **kw):
         """Convert the rdataset into DNS master file format.
 
-        @see: L{dns.name.Name.choose_relativity} for more information
-        on how I{origin} and I{relativize} determine the way names
+        See ``dns.name.Name.choose_relativity`` for more information
+        on how *origin* and *relativize* determine the way names
         are emitted.
 
         Any additional keyword arguments are passed on to the rdata
-        to_text() method.
-        
-        @param name: If name is not None, emit a RRs with I{name} as
-        the owner name.
-        @type name: dns.name.Name object
-        @param origin: The origin for relative names, or None.
-        @type origin: dns.name.Name object
-        @param relativize: True if names should names be relativized
-        @type relativize: bool"""
-        if not name is None:
+        ``to_text()`` method.
+
+        *name*, a ``dns.name.Name``.  If name is not ``None``, emit RRs with
+        *name* as the owner name.
+
+        *origin*, a ``dns.name.Name`` or ``None``, the origin for relative
+        names.
+
+        *relativize*, a ``bool``.  If ``True``, names will be relativized
+        to *origin*.
+        """
+
+        if name is not None:
             name = name.choose_relativity(origin, relativize)
             ntext = str(name)
             pad = ' '
         else:
             ntext = ''
             pad = ''
-        s = StringIO.StringIO()
-        if not override_rdclass is None:
+        s = io.StringIO()
+        if override_rdclass is not None:
             rdclass = override_rdclass
         else:
             rdclass = self.rdclass
@@ -203,15 +214,16 @@ class Rdataset(dns.set.Set):
             # some dynamic updates, so we don't need to print out the TTL
             # (which is meaningless anyway).
             #
-            print >> s, '%s%s%s %s' % (ntext, pad,
-                                       dns.rdataclass.to_text(rdclass),
-                                       dns.rdatatype.to_text(self.rdtype))
+            s.write('{}{}{} {}\n'.format(ntext, pad,
+                                         dns.rdataclass.to_text(rdclass),
+                                         dns.rdatatype.to_text(self.rdtype)))
         else:
             for rd in self:
-                print >> s, '%s%s%d %s %s %s' % \
-                      (ntext, pad, self.ttl, dns.rdataclass.to_text(rdclass),
-                       dns.rdatatype.to_text(self.rdtype),
-                       rd.to_text(origin=origin, relativize=relativize, **kw))
+                s.write('%s%s%d %s %s %s\n' %
+                        (ntext, pad, self.ttl, dns.rdataclass.to_text(rdclass),
+                         dns.rdatatype.to_text(self.rdtype),
+                         rd.to_text(origin=origin, relativize=relativize,
+                         **kw)))
         #
         # We strip off the final \n for the caller's convenience in printing
         #
@@ -221,20 +233,30 @@ class Rdataset(dns.set.Set):
                 override_rdclass=None, want_shuffle=True):
         """Convert the rdataset to wire format.
 
-        @param name: The owner name of the RRset that will be emitted
-        @type name: dns.name.Name object
-        @param file: The file to which the wire format data will be appended
-        @type file: file
-        @param compress: The compression table to use; the default is None.
-        @type compress: dict
-        @param origin: The origin to be appended to any relative names when
-        they are emitted.  The default is None.
-        @returns: the number of records emitted
-        @rtype: int
+        *name*, a ``dns.name.Name`` is the owner name to use.
+
+        *file* is the file where the name is emitted (typically a
+        BytesIO file).
+
+        *compress*, a ``dict``, is the compression table to use.  If
+        ``None`` (the default), names will not be compressed.
+
+        *origin* is a ``dns.name.Name`` or ``None``.  If the name is
+        relative and origin is not ``None``, then *origin* will be appended
+        to it.
+
+        *override_rdclass*, an ``int``, is used as the class instead of the
+        class of the rdataset.  This is useful when rendering rdatasets
+        associated with dynamic updates.
+
+        *want_shuffle*, a ``bool``.  If ``True``, then the order of the
+        Rdatas within the Rdataset will be shuffled before rendering.
+
+        Returns an ``int``, the number of records emitted.
         """
 
-        if not override_rdclass is None:
-            rdclass =  override_rdclass
+        if override_rdclass is not None:
+            rdclass = override_rdclass
             want_shuffle = False
         else:
             rdclass = self.rdclass
@@ -264,66 +286,72 @@ class Rdataset(dns.set.Set):
                 file.write(stuff)
                 file.seek(0, 2)
             return len(self)
-    
+
     def match(self, rdclass, rdtype, covers):
-        """Returns True if this rdataset matches the specified class, type,
-        and covers"""
+        """Returns ``True`` if this rdataset matches the specified class,
+        type, and covers.
+        """
         if self.rdclass == rdclass and \
            self.rdtype == rdtype and \
            self.covers == covers:
             return True
         return False
 
-def from_text_list(rdclass, rdtype, ttl, text_rdatas):
+
+def from_text_list(rdclass, rdtype, ttl, text_rdatas, idna_codec=None):
     """Create an rdataset with the specified class, type, and TTL, and with
     the specified list of rdatas in text format.
 
-    @rtype: dns.rdataset.Rdataset object
+    *idna_codec*, a ``dns.name.IDNACodec``, specifies the IDNA
+    encoder/decoder to use; if ``None``, the default IDNA 2003
+    encoder/decoder is used.
+
+    Returns a ``dns.rdataset.Rdataset`` object.
     """
 
-    if isinstance(rdclass, str):
-        rdclass = dns.rdataclass.from_text(rdclass)
-    if isinstance(rdtype, str):
-        rdtype = dns.rdatatype.from_text(rdtype)
+    rdclass = dns.rdataclass.RdataClass.make(rdclass)
+    rdtype = dns.rdatatype.RdataType.make(rdtype)
     r = Rdataset(rdclass, rdtype)
     r.update_ttl(ttl)
     for t in text_rdatas:
-        rd = dns.rdata.from_text(r.rdclass, r.rdtype, t)
+        rd = dns.rdata.from_text(r.rdclass, r.rdtype, t, idna_codec=idna_codec)
         r.add(rd)
     return r
-    
+
+
 def from_text(rdclass, rdtype, ttl, *text_rdatas):
     """Create an rdataset with the specified class, type, and TTL, and with
     the specified rdatas in text format.
-    
-    @rtype: dns.rdataset.Rdataset object
+
+    Returns a ``dns.rdataset.Rdataset`` object.
     """
 
     return from_text_list(rdclass, rdtype, ttl, text_rdatas)
 
+
 def from_rdata_list(ttl, rdatas):
     """Create an rdataset with the specified TTL, and with
     the specified list of rdata objects.
-    
-    @rtype: dns.rdataset.Rdataset object
+
+    Returns a ``dns.rdataset.Rdataset`` object.
     """
 
     if len(rdatas) == 0:
-        raise ValueError, "rdata list must not be empty"
+        raise ValueError("rdata list must not be empty")
     r = None
     for rd in rdatas:
         if r is None:
             r = Rdataset(rd.rdclass, rd.rdtype)
             r.update_ttl(ttl)
-            first_time = False
         r.add(rd)
     return r
-    
+
+
 def from_rdata(ttl, *rdatas):
     """Create an rdataset with the specified TTL, and with
     the specified rdata objects.
-    
-    @rtype: dns.rdataset.Rdataset object
+
+    Returns a ``dns.rdataset.Rdataset`` object.
     """
 
     return from_rdata_list(ttl, rdatas)
