@@ -1,4 +1,6 @@
-# Copyright (C) 2003-2007 Nominum, Inc.
+# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license
+
+# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -15,6 +17,10 @@
 
 import dns.rdtypes.nsbase
 
+
 class DNAME(dns.rdtypes.nsbase.UncompressedNS):
+
     """DNAME record"""
-    pass
+
+    def _to_wire(self, file, compress=None, origin=None, canonicalize=False):
+        self.target.to_wire(file, None, origin, canonicalize)
