@@ -1,4 +1,6 @@
-# Copyright (C) 2003-2007, 2009-2011 Nominum, Inc.
+# Copyright (C) Dnspython Contributors, see LICENSE for text of ISC license
+
+# Copyright (C) 2003-2017 Nominum, Inc.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose with or without fee is hereby granted,
@@ -17,41 +19,44 @@
 
 import dns.exception
 
+
 class BadTTL(dns.exception.SyntaxError):
-    pass
+    """DNS TTL value is not well-formed."""
+
 
 def from_text(text):
     """Convert the text form of a TTL to an integer.
 
     The BIND 8 units syntax for TTLs (e.g. '1w6d4h3m10s') is supported.
 
-    @param text: the textual TTL
-    @type text: string
-    @raises dns.ttl.BadTTL: the TTL is not well-formed
-    @rtype: int
+    *text*, a ``str``, the textual TTL.
+
+    Raises ``dns.ttl.BadTTL`` if the TTL is not well-formed.
+
+    Returns an ``int``.
     """
 
     if text.isdigit():
-        total = long(text)
+        total = int(text)
     else:
         if not text[0].isdigit():
             raise BadTTL
-        total = 0L
-        current = 0L
+        total = 0
+        current = 0
         for c in text:
             if c.isdigit():
                 current *= 10
-                current += long(c)
+                current += int(c)
             else:
                 c = c.lower()
                 if c == 'w':
-                    total += current * 604800L
+                    total += current * 604800
                 elif c == 'd':
-                    total += current * 86400L
+                    total += current * 86400
                 elif c == 'h':
-                    total += current * 3600L
+                    total += current * 3600
                 elif c == 'm':
-                    total += current * 60L
+                    total += current * 60
                 elif c == 's':
                     total += current
                 else:
@@ -59,6 +64,6 @@ def from_text(text):
                 current = 0
         if not current == 0:
             raise BadTTL("trailing integer")
-    if total < 0L or total > 2147483647L:
+    if total < 0 or total > 2147483647:
         raise BadTTL("TTL should be between 0 and 2^31 - 1 (inclusive)")
     return total
