@@ -24,37 +24,53 @@ import dns.opcode
 class FlagsTestCase(unittest.TestCase):
 
     def test_rcode1(self):
-        self.failUnless(dns.rcode.from_text('FORMERR') == dns.rcode.FORMERR)
+        self.assertEqual(dns.rcode.from_text('FORMERR'), dns.rcode.FORMERR)
 
     def test_rcode2(self):
-        self.failUnless(dns.rcode.to_text(dns.rcode.FORMERR) == "FORMERR")
+        self.assertEqual(dns.rcode.to_text(dns.rcode.FORMERR), "FORMERR")
 
     def test_rcode3(self):
-        self.failUnless(dns.rcode.to_flags(dns.rcode.FORMERR) == (1, 0))
+        self.assertEqual(dns.rcode.to_flags(dns.rcode.FORMERR), (1, 0))
 
     def test_rcode4(self):
-        self.failUnless(dns.rcode.to_flags(dns.rcode.BADVERS) == \
-                        (0, 0x01000000))
+        self.assertEqual(dns.rcode.to_flags(dns.rcode.BADVERS),
+                         (0, 0x01000000))
 
     def test_rcode6(self):
-        self.failUnless(dns.rcode.from_flags(0, 0x01000000) == \
-                        dns.rcode.BADVERS)
+        self.assertEqual(dns.rcode.from_flags(0, 0x01000000),
+                         dns.rcode.BADVERS)
 
     def test_rcode7(self):
-        self.failUnless(dns.rcode.from_flags(5, 0) == dns.rcode.REFUSED)
+        self.assertEqual(dns.rcode.from_flags(5, 0), dns.rcode.REFUSED)
 
     def test_rcode8(self):
         def bad():
             dns.rcode.to_flags(4096)
-        self.failUnlessRaises(ValueError, bad)
+        self.assertRaises(ValueError, bad)
 
     def test_flags1(self):
-        self.failUnless(dns.flags.from_text("RA RD AA QR") == \
-                        dns.flags.QR|dns.flags.AA|dns.flags.RD|dns.flags.RA)
+        self.assertEqual(dns.flags.from_text("RA RD AA QR"),
+                         dns.flags.QR|dns.flags.AA|dns.flags.RD|dns.flags.RA)
 
     def test_flags2(self):
         flags = dns.flags.QR|dns.flags.AA|dns.flags.RD|dns.flags.RA
-        self.failUnless(dns.flags.to_text(flags) == "QR AA RD RA")
+        self.assertEqual(dns.flags.to_text(flags), "QR AA RD RA")
+
+    def test_rcode_badvers(self):
+        rcode = dns.rcode.BADVERS
+        self.assertEqual(rcode.value, 16)
+        self.assertEqual(rcode.name, 'BADVERS')
+        self.assertEqual(dns.rcode.to_text(rcode), 'BADVERS')
+
+    def test_rcode_badsig(self):
+        rcode = dns.rcode.BADSIG
+        self.assertEqual(rcode.value, 16)
+        # Yes, we mean BADVERS on the next line.  BADSIG and BADVERS have
+        # the same code.
+        self.assertEqual(rcode.name, 'BADVERS')
+        self.assertEqual(dns.rcode.to_text(rcode), 'BADVERS')
+        # In TSIG text mode, it should be BADSIG
+        self.assertEqual(dns.rcode.to_text(rcode, True), 'BADSIG')
 
 
 if __name__ == '__main__':
