@@ -26,12 +26,7 @@ import dns.rdataclass
 import dns.rdatatype
 import dns.rrset
 import dns.zone
-from dns._compat import long
 
-import pprint
-
-
-pp = pprint.PrettyPrinter(indent=2)
 
 example_text = """$TTL 1h
 $ORIGIN 0.0.192.IN-ADDR.ARPA.
@@ -133,6 +128,23 @@ ns2 3600 IN A 10.0.0.2
 $GENERATE 27-28 $.2 PTR zlb${-26}.oob
 """
 
+example_text11 = """$TTL 1h
+@ 3600 IN SOA foo bar 1 2 3 4 5
+@ 3600 IN NS ns1
+@ 3600 IN NS ns2
+bar.foo 300 IN MX 0 blaz.foo
+ns1 3600 IN A 10.0.0.1
+ns2 3600 IN A 10.0.0.2
+$GENERATE 27-28 prefix-${0,3}  A 10.0.0.$
+"""
+
+last_ttl_input = """foo 300 mx 10 target.
+$GENERATE 1-10 foo$ CNAME $.0
+@ 3600 IN SOA foo bar 1 2 3 4 5
+@ 3600 IN NS ns1
+@ 3600 IN NS ns2
+"""
+
 def _rdata_sort(a):
     return (a[0], a[2].rdclass, a[2].to_text())
 
@@ -142,12 +154,12 @@ class GenerateTestCase(unittest.TestCase):
     def testFromText(self): # type: () -> None
         def bad(): # type: () -> None
             dns.zone.from_text(example_text, 'example.', relativize=True)
-        self.failUnlessRaises(dns.zone.NoSOA, bad)
+        self.assertRaises(dns.zone.NoSOA, bad)
 
     def testFromText1(self): # type: () -> None
         def bad(): # type: () -> None
             dns.zone.from_text(example_text1, 'example.', relativize=True)
-        self.failUnlessRaises(dns.zone.NoSOA, bad)
+        self.assertRaises(dns.zone.NoSOA, bad)
 
     def testIterateAllRdatas2(self): # type: () -> None
         z = dns.zone.from_text(example_text2, 'example.', relativize=True)
@@ -191,7 +203,7 @@ class GenerateTestCase(unittest.TestCase):
                                     '10.0.0.5'))]
 
         exl.sort(key=_rdata_sort)
-        self.failUnless(l == exl)
+        self.assertEqual(l, exl)
 
     def testIterateAllRdatas3(self): # type: () -> None
         z = dns.zone.from_text(example_text3, 'example.', relativize=True)
@@ -231,48 +243,48 @@ class GenerateTestCase(unittest.TestCase):
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.8'))]
         exl.sort(key=_rdata_sort)
-        self.failUnless(l == exl)
+        self.assertEqual(l, exl)
     def testGenerate1(self): # type: () -> None
         z = dns.zone.from_text(example_text4, 'example.', relativize=True)
         l = list(z.iterate_rdatas())
         l.sort(key=_rdata_sort)
         exl = [(dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns1')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns2')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.SOA,
                                     'foo bar 1 2 3 4 5')),
                (dns.name.from_text('bar.foo', None),
-                long(300),
+                300,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.MX,
                                     '0 blaz.foo')),
                (dns.name.from_text('ns1', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.1')),
                (dns.name.from_text('ns2', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.2')),
 
                 (dns.name.from_text('wp-db01.services.mozilla.com', None),
-                    long(0),
+                    0,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.CNAME,
                                     'SERVER.FOOBAR.')),
 
                 (dns.name.from_text('wp-db02.services.mozilla.com', None),
-                    long(0),
+                    0,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.CNAME,
                                     'SERVER.FOOBAR.')),
 
                 (dns.name.from_text('wp-db03.services.mozilla.com', None),
-                    long(0),
+                    0,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.CNAME,
                                     'SERVER.FOOBAR.'))]
         exl.sort(key=_rdata_sort)
@@ -283,43 +295,43 @@ class GenerateTestCase(unittest.TestCase):
         l = list(z.iterate_rdatas())
         l.sort(key=_rdata_sort)
         exl = [(dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns1')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns2')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.SOA,
                                     'foo bar 1 2 3 4 5')),
                (dns.name.from_text('bar.foo', None),
-                long(300),
+                300,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.MX,
                                     '0 blaz.foo')),
                (dns.name.from_text('ns1', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.1')),
                (dns.name.from_text('ns2', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.2')),
 
-                (dns.name.from_text('wp-db21.services.mozilla.com', None), long(0),
+                (dns.name.from_text('wp-db21.services.mozilla.com', None), 0,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.CNAME,
                                     'SERVER.FOOBAR.')),
 
-                (dns.name.from_text('wp-db22.services.mozilla.com', None), long(0),
+                (dns.name.from_text('wp-db22.services.mozilla.com', None), 0,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.CNAME,
                                     'SERVER.FOOBAR.')),
 
-                (dns.name.from_text('wp-db23.services.mozilla.com', None), long(0),
+                (dns.name.from_text('wp-db23.services.mozilla.com', None), 0,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.CNAME,
                                     'SERVER.FOOBAR.'))]
         exl.sort(key=_rdata_sort)
-        self.failUnless(l == exl)
+        self.assertEqual(l, exl)
 
     def testGenerate3(self): # type: () -> None
         z = dns.zone.from_text(example_text6, 'example.', relativize=True)
@@ -327,176 +339,255 @@ class GenerateTestCase(unittest.TestCase):
         l.sort(key=_rdata_sort)
 
         exl = [(dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns1')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns2')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.SOA,
                                     'foo bar 1 2 3 4 5')),
                (dns.name.from_text('bar.foo', None),
-                long(300),
+                300,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.MX,
                                     '0 blaz.foo')),
                (dns.name.from_text('ns1', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.1')),
                (dns.name.from_text('ns2', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.2')),
-                (dns.name.from_text('wp-db21.services.mozilla.com', None), long(0),
+                (dns.name.from_text('wp-db21.services.mozilla.com', None), 0,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.CNAME,
                                     'SERVER.FOOBAR.')),
 
-                (dns.name.from_text('wp-db22.services.mozilla.com', None), long(0),
+                (dns.name.from_text('wp-db22.services.mozilla.com', None), 0,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.CNAME,
                                     'SERVER.FOOBAR.')),
 
-                (dns.name.from_text('wp-db23.services.mozilla.com', None), long(0),
+                (dns.name.from_text('wp-db23.services.mozilla.com', None), 0,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.CNAME,
                                     'SERVER.FOOBAR.'))]
         exl.sort(key=_rdata_sort)
-        self.failUnless(l == exl)
+        self.assertEqual(l, exl)
 
     def testGenerate4(self): # type: () -> None
         z = dns.zone.from_text(example_text7, 'example.', relativize=True)
         l = list(z.iterate_rdatas())
         l.sort(key=_rdata_sort)
         exl = [(dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns1')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns2')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.SOA,
                                     'foo bar 1 2 3 4 5')),
                (dns.name.from_text('bar.foo', None),
-                long(300),
+                300,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.MX,
                                     '0 blaz.foo')),
                (dns.name.from_text('ns1', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.1')),
                (dns.name.from_text('ns2', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.2')),
 
-                (dns.name.from_text('sync1.db', None), long(3600),
+                (dns.name.from_text('sync1.db', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.10.16.0')),
 
-                (dns.name.from_text('sync2.db', None), long(3600),
+                (dns.name.from_text('sync2.db', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.10.16.0')),
 
-                (dns.name.from_text('sync3.db', None), long(3600),
+                (dns.name.from_text('sync3.db', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.10.16.0'))]
         exl.sort(key=_rdata_sort)
-        self.failUnless(l == exl)
+        self.assertEqual(l, exl)
 
     def testGenerate6(self): # type: () -> None
         z = dns.zone.from_text(example_text9, 'example.', relativize=True)
         l = list(z.iterate_rdatas())
         l.sort(key=_rdata_sort)
         exl = [(dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns1')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns2')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.SOA,
                                     'foo bar 1 2 3 4 5')),
                (dns.name.from_text('bar.foo', None),
-                long(300),
+                300,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.MX,
                                     '0 blaz.foo')),
                (dns.name.from_text('ns1', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.1')),
                (dns.name.from_text('ns2', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.2')),
 
-                (dns.name.from_text('wp-db01', None), long(3600),
+                (dns.name.from_text('wp-db01', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.10.16.0')),
-                (dns.name.from_text('wp-db02', None), long(3600),
+                (dns.name.from_text('wp-db02', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.10.16.0')),
 
-                (dns.name.from_text('sync1.db', None), long(3600),
+                (dns.name.from_text('sync1.db', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.10.16.0')),
 
-                (dns.name.from_text('sync2.db', None), long(3600),
+                (dns.name.from_text('sync2.db', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.10.16.0')),
 
-                (dns.name.from_text('sync3.db', None), long(3600),
+                (dns.name.from_text('sync3.db', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.10.16.0'))]
         exl.sort(key=_rdata_sort)
-        self.failUnless(l == exl)
+        self.assertEqual(l, exl)
 
     def testGenerate7(self): # type: () -> None
         z = dns.zone.from_text(example_text10, 'example.', relativize=True)
         l = list(z.iterate_rdatas())
         l.sort(key=_rdata_sort)
         exl = [(dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns1')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
                                     'ns2')),
                (dns.name.from_text('@', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.SOA,
                                     'foo bar 1 2 3 4 5')),
                (dns.name.from_text('bar.foo', None),
-                long(300),
+                300,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.MX,
                                     '0 blaz.foo')),
                (dns.name.from_text('ns1', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.1')),
                (dns.name.from_text('ns2', None),
-                long(3600),
+                3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
                                     '10.0.0.2')),
 
-                (dns.name.from_text('27.2', None), long(3600),
+                (dns.name.from_text('27.2', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.PTR,
                                     'zlb1.oob')),
 
-                (dns.name.from_text('28.2', None), long(3600),
+                (dns.name.from_text('28.2', None), 3600,
                 dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.PTR,
                                     'zlb2.oob'))]
 
         exl.sort(key=_rdata_sort)
-        self.failUnless(l == exl)
+        self.assertEqual(l, exl)
 
+    def testGenerate8(self): # type: () -> None
+        z = dns.zone.from_text(example_text11, 'example.', relativize=True)
+        l = list(z.iterate_rdatas())
+        l.sort(key=_rdata_sort)
+        exl = [(dns.name.from_text('@', None),
+                3600,
+                dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
+                                    'ns1')),
+               (dns.name.from_text('@', None),
+                3600,
+                dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.NS,
+                                    'ns2')),
+               (dns.name.from_text('@', None),
+                3600,
+                dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.SOA,
+                                    'foo bar 1 2 3 4 5')),
+               (dns.name.from_text('bar.foo', None),
+                300,
+                dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.MX,
+                                    '0 blaz.foo')),
+
+               (dns.name.from_text('prefix-027', None), 3600,
+                dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
+                                    '10.0.0.27')),
+
+               (dns.name.from_text('prefix-028', None), 3600,
+                dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
+                                    '10.0.0.28')),
+
+               (dns.name.from_text('ns1', None),
+                3600,
+                dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
+                                    '10.0.0.1')),
+               (dns.name.from_text('ns2', None),
+                3600,
+                dns.rdata.from_text(dns.rdataclass.IN, dns.rdatatype.A,
+                                    '10.0.0.2'))]
+
+        exl.sort(key=_rdata_sort)
+        self.assertEqual(l, exl)
+
+    def testNoOrigin(self):
+        def bad():
+            dns.zone.from_text('$GENERATE 1-10 fooo$ CNAME $.0')
+        self.assertRaises(dns.zone.UnknownOrigin, bad)
+
+    def testBadRdata(self):
+        def bad():
+            dns.zone.from_text('$GENERATE 1-10 fooo$ CNAME 10 $.0', 'example')
+        self.assertRaises(dns.exception.SyntaxError, bad)
+
+    def testUsesLastTTL(self):
+        z = dns.zone.from_text(last_ttl_input, 'example')
+        rrs = z.find_rrset('foo9', 'CNAME')
+        self.assertEqual(rrs.ttl, 300)
+
+    def testClassMismatch(self):
+        def bad():
+            dns.zone.from_text('$GENERATE 1-10 fooo$ CH CNAME $.0', 'example')
+        self.assertRaises(dns.exception.SyntaxError, bad)
+
+    def testUnknownRdatatype(self):
+        def bad():
+            dns.zone.from_text('$GENERATE 1-10 fooo$ BOGUSTYPE $.0', 'example')
+        self.assertRaises(dns.exception.SyntaxError, bad)
+
+    def testBadAndDangling(self):
+        def bad1():
+            dns.zone.from_text('$GENERATE bogus fooo$ CNAME $.0',
+                               'example.')
+        self.assertRaises(dns.exception.SyntaxError, bad1)
+        def bad2():
+            dns.zone.from_text('$GENERATE 1-10',
+                               'example.')
+        self.assertRaises(dns.exception.SyntaxError, bad2)
+        def bad3():
+            dns.zone.from_text('$GENERATE 1-10 foo$',
+                               'example.')
+        self.assertRaises(dns.exception.SyntaxError, bad3)
 
 if __name__ == '__main__':
     unittest.main()
